!+
! Module: rgssources_init
!
! Authors:  Ian Stewart - SSC/LUX
!        :  Masaaki Sakano - SSC/LUX
! 
! Description: Contains the type specification and the initialisation functions for the source data structure.
! 
! $Id: rgssources_init.f90,v 1.2 2000/08/11 13:08:45 pvideler Exp $
!- 
module rgssources_init

! You should write the module name in the 'use' statement, the package name
! (and version number) in the DEPEND file, and the * from the lib*.a name in
! <package>/src/Makefile.
  use dal

  implicit none 

  private
 
  public initSrcDataStructure!!!!, sourceType
!  public labelsize, myInt16null, epicBandNum, myInt32null, myReal32null
!  public myOUTOFFOV, myCONFUSED, myBADBAND

  interface initSrcDataStructure
    module procedure initSrcDataStructureScalar, initSrcDataStructureVector
  end interface

  integer(int16), parameter, public ::&
    MAXFITSCOMMENTSIZE = 80,&
    labelsize = 20,&
    myInt16null = -999,&
    OUTOFFOV_BIT = 0,&
    CONFUSED_BIT = 1,&
    BADBAND_BIT  = 2,&
    WIDESRC_BIT  = 3
    ! epicBandNum = 2,&		! Now, it's specified by user ==> epicBandIds
  integer(int16), allocatable, public ::&
    epicBandIds(:)
  integer(int32), parameter, public ::&
    myInt32null = -999,&
    OUTOFFOV_MASK = ibset(int(0, int32), OUTOFFOV_BIT),&
    CONFUSED_MASK = ibset(int(0, int32), CONFUSED_BIT),&
    BADBAND_MASK  = ibset(int(0, int32), BADBAND_BIT),&
    WIDESRC_MASK  = ibset(int(0, int32), WIDESRC_BIT)
  real(single), parameter, public ::&
    myReal32null = -999.0
  character(len=labelsize), parameter, public ::&
    proposalLabel = 'PROPOSAL',&
    onAxisLabel = 'ONAXIS'
  character(len=MAXFITSCOMMENTSIZE), parameter, public ::&
    COMMEPICEXPID   = 'EPIC source list Instrument',&
    COMMEPICCONTENT = 'EPIC source list Content name',&
    COMMEPICBANDNUM = 'EPIC source list Band-ID(s)',&
    COMMEPICFILTER  = 'EPIC source list Filter used',&
    COMMPRIMESRC    = 'Prime Source (for spectrum extraction)'
  character(len=4), parameter, public ::&
    GLcolNameOfFlag = 'FLAG'

  type, public :: sourceType
    integer(int16) ::&
      index
    integer ::&
      flagLocal	= -999	! Local flag (negative: uninitialised, 0: normal, positive: something is wrong)
    integer(int32) ::&
      flag,&	! output to the column 'FLAG' (e.g. set at OUTOFFOV_BIT)
      confusionSet,&
      epicFileAttrNum ! has to be 32 rather than 16 so can set an integer
      ! null using the dal call setNullValue.
    character(len=labelsize) ::&
      label
    real(double) ::&
      raDeg,&
      decDeg
    real(single) ::&
      rate,&
counts,&
      bkgRate,&
bkgFlux,&		! in counts per square arc second
      deltaDispArcmin,&
      deltaXdspArcmin,&
      fovRdeg,&
      fovPhiDeg,&
      confusion
    logical(bool) ::&
      process,&
      bkgExclude,&
      fixedOnSky
  endType sourceType

  type, public :: srcEnvelopeType
    real(double) ::&
      thetaArcsec,&
      EeV
    real(single) ::&
      peakFlux,&
      bkgFlux,&
      longAxisArcsec,&
      shortAxisArcsec
  end type srcEnvelopeType

  type, public :: instWeightType
    integer(int16) :: idInst
    real(single) :: weight
  end type instWeightType

contains 

  !---------------------------------------------------------------------
  subroutine initSrcDataStructureScalar(srcData)
  !---------------------------------------------------------------------

    implicit none 

    type(sourceType), intent(out) ::&
      srcData

    type(sourceType) ::&
      srcDataVector(1)

    !-------------------------------------------------------------------
    ! Start:

    call initSrcDataStructureVector(srcDataVector)
    srcData = srcDataVector(1)

  end subroutine initSrcDataStructureScalar

  !---------------------------------------------------------------------
  subroutine initSrcDataStructureVector(srcData)
  !---------------------------------------------------------------------

    implicit none 

    type(sourceType), dimension(:), intent(out) ::&
      srcData

    !-------------------------------------------------------------------
    ! Start:

    srcData%index           = myInt16null
    srcData%label           = ''
    srcData%raDeg           = myReal32null
    srcData%decDeg          = myReal32null
    srcData%rate            = 0.0
    srcData%bkgRate         = 0.0 ! get values from epic list, then give some average value to the non-epic sources
    srcData%deltaDispArcmin = myReal32null
    srcData%deltaXdspArcmin = myReal32null
    srcData%fovPhiDeg       = myReal32null
    srcData%fovRdeg         = myReal32null
    srcData%confusion       = myReal32null
    srcData%process         = .false.
    srcData%bkgExclude      = .false.
    srcData%fixedOnSky      = .true.
    srcData%epicFileAttrNum = myInt32null
    srcData%flag            = 0
    srcData%confusionSet    = 0

  end subroutine initSrcDataStructureVector
end module rgssources_init






